/**
 * A class that represents a Binary Tree, defined with Nodes
 * @author 	Joel
 * @version  Sp 2013
 */
public class BinaryTree<E>
{
	private static final String SPACE_DELIM = "  "; //for printing

	protected Node<E> root; //The root of the tree
		
	/**
	 * Creates a new binary tree
	 * @param  data        data to go in the root
	 * @param  leftBranch  a tree to attach to the left
	 * @param  rightBranch a tree to attach to the right
	 */
	public BinaryTree(E data, BinaryTree<E> leftBranch, BinaryTree<E> rightBranch)
	{
		root = new Node<E>(data); //take the given data, and make it into the root
		if(leftBranch != null)
			root.left = leftBranch.root; //grab the root from the given trees, make them the child
		if(rightBranch != null)
			root.right = rightBranch.root;
	}

	/**
	 * Creates a new binary tree with no left or right subtrees
	 * @param  data data to go in the root
	 */
	public BinaryTree(E data)
	{
		this(data, null, null); // call previous method
	}

	/**
	 * [BinaryTree description]
	 * @return [description]
	 */
	public BinaryTree()
	{
		this.root = null;
	}
	
	/**
	 * Creates a new binary tree out of the given node
	 * @param  root the note that should become the root
	 */
	protected BinaryTree(Node<E> root)
	{
		this.root = root;
	}	
		
	/**
	 * Returns the data in the root of the tree
	 * @return The data in the root
	 */
	public E getData()
	{
		return root.data;
	}
	
	/**
	 * Prints the tree using a pre-order traversal
	 */
	public void printPreOrder(){
		printPreOrder(root); //call a helper (doesn't need separate name)
		System.out.println();
	}

	/**
	 * Prints the tree use a post-order traversal
	 */
	public void printPostOrder(){
		printPostOrder(root);
		System.out.println();
	}

	/**
	 * Prints the tree using an in-order traversal
	 */
	public void printInOrder(){
		printInOrder(root);
		System.out.println();
	}

	/**
	 * A helper to print the tree using a pre-order traversal
	 * @param localRoot the root of the subtree to print
	 */
	private void printPreOrder(Node<E> localRoot)
	{
		System.out.print(localRoot.data+", ");
		if(localRoot.left != null)
			printPreOrder(localRoot.left);
		if(localRoot.right != null)
			printPreOrder(localRoot.right);
	}

	/**
	 * A helper to print the tree using a post-order traversal
	 * @param localRoot the root of the subtree to print
	 */
	private void printPostOrder(Node<E> localRoot)
	{
		if(localRoot.left != null)
			printPostOrder(localRoot.left);
		if(localRoot.right != null)
			printPostOrder(localRoot.right);
		System.out.print(localRoot.data+", ");
	}

	/**
	 * A helper to print the tree using an in-order traversal
	 * @param localRoot the root of the subtree to print
	 */
	private void printInOrder(Node<E> localRoot)
	{
		if(localRoot.left != null)
			printInOrder(localRoot.left);
		System.out.print(localRoot.data+", ");
		if(localRoot.right != null)
			printInOrder(localRoot.right);
	}

	
	/**
	 * Returns a String representation of this tree
	 */	
	public String toString()
	{
		return root.toStringPreOrder("");
	}
	
	/**
	 * Inner class that represents a Node in the tree
	 */
	protected static class Node<E>
	{
		protected E data;
		protected Node<E> left; //left child
		protected Node<E> right; //right child
		
		public Node(E data)
		{
			this.data = data;
		}
		
		public String toString()
		{
			return data.toString();
		}
		
		
		
		/**
		 * Returns a String representation of the subtree represented by this node, produced using a pre-order traversal
		 * @param  currentSpace The current indentation level of the Node
		 * @return              The String representation of the subtree
		 */
		private String toStringPreOrder(String currentSpace)
		{
			String s = "";
		
			s += currentSpace + data + "\n";

			if(left != null)
				s += left.toStringPreOrder(currentSpace+SPACE_DELIM);
			
			if(right != null)
				s += right.toStringPreOrder(currentSpace+SPACE_DELIM);
				
			return s;
		}
		
		/**
		 * Returns a String representation of the subtree represented by this node, produced using a post-order traversal
		 * @param  currentSpace The current indentation level of the Node
		 * @return              The String representation of the subtree
		 */
		private String toStringPostOrder(String currentSpace)
		{
			String s = "";
			
			if(left != null)
				s += left.toStringPostOrder(currentSpace+SPACE_DELIM);
			
			if(right != null)
				s += right.toStringPostOrder(currentSpace+SPACE_DELIM);

			s += currentSpace + data + "\n";
			
			return s;
		}
	
		/**
		 * Returns a String representation of the subtree represented by this node, produced using an in-order traversal
		 * @param  currentSpace The current indentation level of the Node
		 * @return              The String representation of the subtree
		 */
		private String toStringInOrder(String currentSpace)
		{
			String s = "";
			
			if(left != null)
				s += left.toStringInOrder(currentSpace+SPACE_DELIM);

			s += currentSpace + data + "\n";
			
			if(right != null)
				s += right.toStringInOrder(currentSpace+SPACE_DELIM);

			return s;
		}
	}	
}
